package org.jeecg.modules.bytedance.advertise.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceCreativeWordPackage;
import org.jeecg.modules.bytedance.advertise.entity.ByteDanceGeneralCopywriter;
import org.jeecg.modules.bytedance.advertise.service.IByteDanceGeneralCopywriterService;
import org.jeecg.modules.bytedance.advertise.service.impl.ByteDanceGeneralCopywriterServiceImpl;
import org.jeecg.modules.bytedance.common.utils.Check;
import org.jeecg.modules.bytedance.common.utils.ExcelUtil;
import org.jeecg.modules.bytedance.common.utils.StringUtils;
import org.jeecg.modules.system.service.ISysRoleService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLDecoder;
import java.text.DecimalFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 头条通用文案上传接口
 * @author jeecg-boot
 * @date   2021-04-20
 * @version V1.0
 */
@RestController
@RequestMapping("/advertiser/bytedanceGeneralCopywriter")
@Api(tags="头条通用文案")
@Slf4j
public class BytedanceGeneralCopywriterController {
    @Autowired
    private IByteDanceGeneralCopywriterService byteDanceGeneralCopywriterService;
    @Autowired
    private ISysRoleService sysRoleService;


    @ApiOperation(value="通用文案库-上传excel", notes="通用文案库-上传excel")
    @PostMapping("/loadExcel")
    public Result<ByteDanceGeneralCopywriter> loadExcel(
            @RequestParam("accountId") String accountId,
            @RequestParam("excelFile") MultipartFile excelFile){
        Result<ByteDanceGeneralCopywriter> result = new Result<>();
        List<ByteDanceGeneralCopywriter> byteDanceGeneralCopywriterList = new ArrayList<ByteDanceGeneralCopywriter>();
        String[] accountIds = new String[]{accountId};
        if(accountId.contains(",")){
            accountIds = accountId.split(",");
        }
        try {
            List<String> list = ExcelUtil.readExcel(excelFile);

            for (String accountIdStr:accountIds) {
                for (String str: list) {
                    ByteDanceGeneralCopywriter byteDanceGeneralCopywriter = new ByteDanceGeneralCopywriter(Long.parseLong(accountIdStr),str);
                    int len = StringUtils.lengthAsSlogan(str);
                    int left = str.indexOf("{");
                    int right = str.indexOf("}");
                    if(left >= 0 && right >= 0){
                        List<String> strs = new ArrayList<>();
                        ByteDanceCreativeWordPackage creativeWordPackage = byteDanceGeneralCopywriterService.selectCreativeWordPackageByName(str.substring(left+1, right));
                        strs.add(creativeWordPackage.getCreativeWordId().toString());
                        len = len - (right - left) + creativeWordPackage.getMaxWordLen();
                        String nextStr = str.substring(right+1,str.length());
                        left = nextStr.indexOf("{");
                        right = nextStr.indexOf("}");
                        if(left >= 0 && right >= 0){
                            ByteDanceCreativeWordPackage creativeWordPackage2 = byteDanceGeneralCopywriterService.selectCreativeWordPackageByName(nextStr.substring(left+1, right));
                            strs.add(creativeWordPackage2.getCreativeWordId().toString());
                            len = len - (right - left) + creativeWordPackage2.getMaxWordLen();
                        }
                        byteDanceGeneralCopywriter.setCreativeWordIds(Arrays.toString(strs.toArray()).replaceAll(" ",""));
                    }
                    //校验标题长度，长度小于5和大于30的全部过滤
                    if(len<5 || len>30){
                        continue;
                    }

                    //截取{}中的内容，匹配动态词包id，拼接数组存储
                    byteDanceGeneralCopywriterList.add(byteDanceGeneralCopywriter);
                }
            }


            byteDanceGeneralCopywriterService.saveBatch(byteDanceGeneralCopywriterList);
            result.success("添加成功"+byteDanceGeneralCopywriterList.size()+"条"+",失败"+(list.size() * accountIds.length - byteDanceGeneralCopywriterList.size())+"条");
            result.setCode(0);
        } catch (IOException e) {
            result.error500("动态词包格式错误，请重新上传！");
        }
        return result;
    }


    /**
     * 分页列表查询
     * @param byteDanceGeneralCopywriter
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value="通用文案库-分页列表查询", notes="通用文案库-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceGeneralCopywriter>> queryPageList(ByteDanceGeneralCopywriter byteDanceGeneralCopywriter,
                                                                   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                                                   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<ByteDanceGeneralCopywriter>> result = new Result<>();
        QueryWrapper<ByteDanceGeneralCopywriter> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceGeneralCopywriter, req.getParameterMap());
        Page<ByteDanceGeneralCopywriter> page = new Page<ByteDanceGeneralCopywriter>(pageNo, pageSize);
        IPage<ByteDanceGeneralCopywriter> pageList = byteDanceGeneralCopywriterService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @ApiOperation(value="通用文案报表展示", notes="通用文案报表展示")
    @GetMapping(value = "/queryPageListByText")
    public Result queryPageListByText(
                                      @RequestParam(value = "userId",defaultValue = "") String userId,
                                      @RequestParam(value = "accountId",defaultValue = "") String accountId,
                                      @RequestParam(value = "keyWord",defaultValue = "") String keyWord,
                                      @RequestParam(value = "startTime",defaultValue = "") String startTime,
                                      @RequestParam(value = "endTime",defaultValue = "") String endTime,
                                      @RequestParam(value = "pageNo", defaultValue="1") Integer pageNo,
                                      @RequestParam(value = "pageSize", defaultValue="10") Integer pageSize
    ) {
        try {

            String roleCode = sysRoleService.getRoleCodeByUserId(userId);
            if (Check.isNull(roleCode)){
                return Result.errorMsg("请输入正确的用户id。");
            }
            //角色为 管理员并且projectId是空表示：全量查询
            if ("admin".equals(roleCode) && Check.isNull(accountId)) {
                accountId = "admin";	//传递projectId为admin
            }

            return byteDanceGeneralCopywriterService.getGeneralCopywriterReport(userId,accountId,keyWord, startTime,endTime,pageNo,pageSize);
        }catch (Exception e){
            log.error("通用文案数据-获取文案维度创意报表数据异常",e);
            return Result.error("请求失败，请联系开发人员!");
        }
    }

    /**
     *   添加
     * @param byteDanceGeneralCopywriter
     * @return
     */
    @ApiOperation(value="通用文案库-添加", notes="通用文案库-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceGeneralCopywriter> add(@RequestBody ByteDanceGeneralCopywriter byteDanceGeneralCopywriter) {
        Result<ByteDanceGeneralCopywriter> result = new Result<>();
        try {
            byteDanceGeneralCopywriterService.save(byteDanceGeneralCopywriter);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(),e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     *   批量添加
     * @param byteDanceGeneralCopywriterList
     * @return
     */
    @ApiOperation(value="通用文案库-批量添加", notes="通用文案库-批量添加")
    @PostMapping(value = "/addList")
    public Result<List> add(@RequestBody List<ByteDanceGeneralCopywriter> byteDanceGeneralCopywriterList) {
        Result<List> result = new Result<>();
        try {
            byteDanceGeneralCopywriterService.saveBatch(byteDanceGeneralCopywriterList);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(),e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     *  编辑
     * @param byteDanceGeneralCopywriter
     * @return
     */
    @ApiOperation(value="通用文案库-编辑", notes="通用文案库-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceGeneralCopywriter> edit(@RequestBody ByteDanceGeneralCopywriter byteDanceGeneralCopywriter) {
        Result<ByteDanceGeneralCopywriter> result = new Result<ByteDanceGeneralCopywriter>();
        ByteDanceGeneralCopywriter byteDanceGeneralCopywriterEntity = byteDanceGeneralCopywriterService.getById(byteDanceGeneralCopywriter.getId());
        if(byteDanceGeneralCopywriterEntity==null) {
            result.error500("未找到对应实体");
        }else {
            boolean ok = byteDanceGeneralCopywriterService.updateById(byteDanceGeneralCopywriter);
            if(ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     *   通过id删除
     * @param id
     * @return
     */
    @ApiOperation(value="通用文案库-通过id删除", notes="通用文案库-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name="id") String id) {
        try {
            byteDanceGeneralCopywriterService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败",e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     *  批量删除
     * @param ids
     * @return
     */
    @ApiOperation(value="通用文案库-批量删除", notes="通用文案库-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceGeneralCopywriter> deleteBatch(@RequestParam(name="ids") String ids) {
        Result<ByteDanceGeneralCopywriter> result = new Result<>();
        if(ids==null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        }else {
            this.byteDanceGeneralCopywriterService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     * @param id
     * @return
     */
    @ApiOperation(value="通用文案库-通过id查询", notes="通用文案库-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceGeneralCopywriter> queryById(@RequestParam(name="id",required=true) String id) {
        Result<ByteDanceGeneralCopywriter> result = new Result<>();
        ByteDanceGeneralCopywriter byteDanceGeneralCopywriter = byteDanceGeneralCopywriterService.getById(id);
        if(byteDanceGeneralCopywriter==null) {
            result.error500("未找到对应实体");
        }else {
            result.setResult(byteDanceGeneralCopywriter);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<ByteDanceGeneralCopywriter> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                ByteDanceGeneralCopywriter byteDanceGeneralCopywriter = JSON.parseObject(deString, ByteDanceGeneralCopywriter.class);
                queryWrapper = QueryGenerator.initQueryWrapper(byteDanceGeneralCopywriter, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<ByteDanceGeneralCopywriter> pageList = byteDanceGeneralCopywriterService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "通用文案库列表");
        mv.addObject(NormalExcelConstants.CLASS, ByteDanceGeneralCopywriter.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("通用文案库列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<ByteDanceGeneralCopywriter> listByteDanceGeneralCopywriters = ExcelImportUtil.importExcel(file.getInputStream(), ByteDanceGeneralCopywriter.class, params);
                byteDanceGeneralCopywriterService.saveBatch(listByteDanceGeneralCopywriters);
                return Result.ok("文件导入成功！数据行数:" + listByteDanceGeneralCopywriters.size());
            } catch (Exception e) {
                log.error(e.getMessage(),e);
                return Result.error("文件导入失败:"+e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
